#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
#include "spi.h"
#include "ST7735R_TFT.h"
#include "DS3231.h"


#define MORSE_LUNG		150
#define MORSE_SCURT		50
#define MORSE_PAUZA		50
#define MAX_LEN			80


char buffer1[MAX_LEN];
static const char *spaces = "                                                                          ";

volatile uint8_t h_alarm;
volatile uint8_t min_alarm;

uint8_t sec, min ,h, m, d, y, s;

volatile int alarm_set = 0;
volatile int alarm_started = 0;
volatile int stopAll = 0;

ISR(PCINT0_vect)
{
	if ((PINA & (1 << PA1)) != 0) {
		//detectare miscare
		
		if (OCR1B != 2 && alarm_started == 1) {
			OCR1B = 2;
			stopAll = 1;
			PORTD |= (1 << PD7);
		}
	} else {
		PORTD &= ~(1 << PD7);

	}
}

ISR(PCINT3_vect)
{
	if ((PIND & (1 << PD5)) == 0)
    {	
		if(alarm_set == 0) {
			OCR1B = 1;
		}
		_delay_ms(250);
	}

	else if ((PIND & (1 << PD4)) == 0)
    {	
		h_alarm = (h_alarm + 1) % 24;
		
		sprintf(buffer1, "%d:%d", h_alarm, min_alarm);
		ST7735R_DrawText(60, 70, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(60, 70, buffer1, 255, 0, 0, 0, 0, 0);
		_delay_ms(250);
	}
	
	else if ((PIND & (1 << PD1)) == 0)
    {	
		min_alarm = (min_alarm + 1) % 60;
		
		sprintf(buffer1, "%d:%d", h_alarm, min_alarm);
		ST7735R_DrawText(60, 70, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(60, 70, buffer1, 255, 0, 0, 0, 0, 0);
		_delay_ms(250);
	} else if ((PIND & (1 << PD0)) == 0)
    {	
		_delay_ms(250);
		
	}
}


/* Genereaza sunetul pentru tipul de ton folosit: lung (150ms) sau scurt (50ms). */
void speaker_morse(int tip_morse) {
	int i;

	/* Alternam starea pinului PA6 la fiecare milisecunda. */
	for (i = 0; i < tip_morse; ++i) {
		PINA = (1 << PA6);
		_delay_loop_2(10000);
	}
}

void timp1_alarma()
{
	int diff = 0;
	DS3231_getTime(&sec, &min, &h, &s, &d, &m, &y);
	diff = 3600 * h + 60 * min + sec -
			3600 * h_alarm - 60 * min_alarm;
	OCR1B = 3;
	if (diff <= 10) {
		OCR1B = 3;
		PORTA |= (1 << PA0);
	} else if (diff > 10 && diff <= 20) {
		OCR1B = 3;
		PORTA &= ~(1 << PA0);
		PORTA |= (1 << PA3);
	} else if (diff > 20 && diff <= 30) {
		OCR1B = 3;
		PORTA &= ~(1 << PA3);
		speaker_morse(MORSE_SCURT);
	} else if (diff > 30) {
		OCR1B = 3;
		PORTA |= (1 << PA3);
		PORTA |= (1 << PA0);
		speaker_morse(MORSE_LUNG);
	}
}



void IO_init()
{
	//buton PD5 = Confirmare alarma
	//buton PD4 = Setare Ora (in mod circular 0-24)
	//buton PD1 = Setare Minut (0-60 cu pas 5)
	DDRD &= ~(1 << PD1) & ~(1 << PD4) & ~(1 << PD5);
	//Rezistenta de pull-up
	PORTD |= (1 << PD1) | (1 << PD4) | (1 << PD5);
	
	//Motor
	DDRA |= (1 << PA0);
	
	//LED
	DDRA |= (1 << PA3);
	
	//Buzzer
	DDRA |= (1 << PA6);
	PORTA &= ~(1 << PA6);
	
	//Senzor de miscare
	DDRA &= ~(1 << PA1);
	
	DDRD |= (1 << PD7);
	
	//Intreruperi pentru butoane si senzor de miscare
	PCICR = 0;
	PCICR |= (1 << PCIE3) | (1 << PCIE0);
	PCMSK3 = 0;
	PCMSK0 = 0;
	PCMSK3 |= (1 << PCINT24) | (1 << PCINT25) 
			| (1 << PCINT28) | (1 << PCINT29);
	PCMSK0 |= (1 << PCINT1);
}

void set_alarm(){
	
	OCR1B = 0;
	uint8_t sec, min, h, m, d, y, s;
	
	h_alarm = 0;
	min_alarm = 0;
	//alarm_started = 0;
	sprintf(buffer1, "Setati alarma");
	ST7735R_DrawText(25, 50, spaces, 255, 255, 0, 0, 0, 0);
	ST7735R_DrawText(20, 100, spaces, 255, 255, 0, 0, 0, 0);
	ST7735R_DrawText(25, 50, buffer1, 255, 0, 0, 0, 0, 0);
	
	sprintf(buffer1, "%d:%d", h_alarm, min_alarm);
	ST7735R_DrawText(60, 70, spaces, 255, 255, 0, 0, 0, 0);
	ST7735R_DrawText(60, 70, buffer1, 255, 0, 0, 0, 0, 0);
	
	while(OCR1B == 0) {
		DS3231_getTime(&sec, &min, &h, &s, &d, &m, &y);
		
		sprintf(buffer1, "ORA: %d:%d:%d", h, min, sec);
		
		ST7735R_DrawText(30, 130, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(30, 130, buffer1, 255, 0, 0, 0, 0, 0);
		_delay_ms(100);
	}
	
	OCR1B = 0;
	
	sprintf(buffer1, "ALARMA SETATA");
	ST7735R_DrawText(25, 50, spaces, 255, 255, 0, 0, 0, 0);
	//ST7735R_DrawText(0, 100, spaces, 255, 255, 0, 0, 0, 0);
	ST7735R_DrawText(25, 50, buffer1, 255, 0, 0, 0, 0, 0);
	
	//sprintf(buffer1, "Ati setat %d:%d", h_alarm, min_alarm);
	//ST7735R_DrawText(50, 100, spaces, 255, 255, 0, 0, 0, 0);
	//ST7735R_DrawText(0, 100, buffer1, 255, 0, 0, 0, 0, 0);
	alarm_set = 1;
	
}

void start_alarm(){
	uint8_t sec, min, h, m, d, y, s;
	//alarm_set = 1;
	//alarm_started = 1;
	//while (OCR1B != 2) {
	while(stopAll == 0) {
		 sprintf(buffer1, "Suna alarma");
		ST7735R_DrawText(20, 50, spaces, 255, 255, 0, 0, 0, 0);
		//ST7735R_DrawText(0, 100, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(20, 50, buffer1, 255, 0, 0, 0, 0, 0);
		
		DS3231_getTime(&sec, &min, &h, &s, &d, &m, &y);
			
			
		sprintf(buffer1, "ORA: %d:%d:%d", h, min, sec);
		
		ST7735R_DrawText(30, 130, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(30, 130, buffer1, 255, 0, 0, 0, 0, 0);
		_delay_ms(100);
		
		timp1_alarma();
		
	}
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA3);
	PORTA &= ~(1 << PA6);
	alarm_set = 0;
	alarm_started = 0;
}




int main(void) {
	SREG = 0;
	SREG |= (1<<7);
	

	SPI_init();
    ST7735R_Begin();
	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
	
	
	
	twi_INIT();
	DS3231_setTime(0,2,1,1,21,5,18);
	IO_init();
	
	while(1){
		set_alarm();
		stopAll = 0;
		sprintf(buffer1, "ALARMA SETATA");
		ST7735R_DrawText(20, 50, spaces, 255, 255, 0, 0, 0, 0);
		//ST7735R_DrawText(0, 100, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(20, 50, buffer1, 255, 0, 0, 0, 0, 0);
		
		sprintf(buffer1, "Alarma la %d:%d", h_alarm, min_alarm);
		ST7735R_DrawText(20, 100, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(20, 100, buffer1, 255, 0, 0, 0, 0, 0);
		
		DS3231_getTime(&sec, &min, &h, &s, &d, &m, &y);
		
		sprintf(buffer1, "ORA: %d:%d:%d", h, min, sec);
		ST7735R_DrawText(30, 130, spaces, 255, 255, 0, 0, 0, 0);
		ST7735R_DrawText(30, 130, buffer1, 255, 0, 0, 0, 0, 0);
		
		while(1) {
			DS3231_getTime(&sec, &min, &h, &s, &d, &m, &y);
			sprintf(buffer1, "ORA: %d:%d:%d", h,min,sec);
			ST7735R_DrawText(30, 130, spaces, 255, 255, 0, 0, 0, 0);
			ST7735R_DrawText(30, 130, buffer1, 255, 0, 0, 0, 0, 0);
			
			if (h_alarm == h && min_alarm == min) {
				alarm_started = 1;
				start_alarm();
				break;
			}
			_delay_ms(100);
		}
	}
	return 0;
}
